import unittest
from dataclasses import dataclass, field

from mag_db.data.column_names_mapping import ColumnNamesMapping
from mag_db.data.db_output import DbOutput
from mag_db.data.result_set_helper import ResultSetHelper


class TestResultSetHelper(unittest.TestCase):
    def setUp(self):
        self.query_result = [
            {'id': 1, 'name': 'Alice', "age": 30},
            {'id': 2, 'name': 'Bob', "age": 25},
            {'id': 3, 'name': 'Charlie', "age": 35}
        ]

        self.db_output = DbOutput()
        self.db_output.column_names = ['id', 'name', 'age']
        column_mapping = ColumnNamesMapping()
        column_mapping.put('id', int, 'id')
        column_mapping.put('name', str, 'name')
        column_mapping.put('age', int, 'age')
        self.db_output.column_name_mapping = column_mapping
        self.db_output.result_class = TestBean

    def test_to_maps(self):
        expected_result = [
            {'id': 1, 'name': 'Alice', 'age': 30},
            {'id': 2, 'name': 'Bob', 'age': 25},
            {'id': 3, 'name': 'Charlie', 'age': 35}
        ]
        result = ResultSetHelper.to_maps(self.query_result, self.db_output)
        self.assertEqual(result, expected_result)

    def test_to_beans(self):
        expected_result = [
            TestBean(id=1, name='Alice', age=30),
            TestBean(id=2, name='Bob', age=25),
            TestBean(id=3, name='Charlie', age=35)
        ]
        result = ResultSetHelper.to_beans(self.query_result, self.db_output)
        self.assertEqual(result, expected_result)
        print(result)

@dataclass
class TestBean:
    id: int = field()
    name: str = field()
    age: int = field()

if __name__ == '__main__':
    unittest.main()



from mag_tools.model.base_enum import BaseEnum


class OperatorType(BaseEnum):
    CREATE_TABLE = ("CREATE TABLE", "创建表")
    DROP_TABLE = ('DROP TABLE IF EXISTS', "删除表")
    INSERT = ("INSERT INTO", "插入数据")
    DELETE = ("DELETE FROM", "删除数据")
    UPDATE = ("UPDATE", "更新数据")
    SELECT = ("SELECT", "查询数据")
    ALTER_TABLE = ("ALTER TABLE", "修改表")
    LOAD_DATA = ("LOAD DATA", "加载数据")
    VALUES = ("VALUES", "值")
    FROM = ("FROM", "从")
    SET = ("SET", "设置")
    ADD = ("ADD", "添加")
    DROP_COLUMN = ("DROP COLUMN", "删除列")
    WHERE = ("WHERE", "条件")
    NOT_IN = ("NOT IN", "不在")
    IN = ("IN", "在")
    DISTINCT = ("SELECT DISTINCT", "选择不同")
    GROUP_BY = ("GROUP BY", "分组")
    GROUP = ("GROUP", "分组")
    ORDER_BY = ("ORDER BY", "排序")
    ORDER = ("ORDER", "排序")
    HAVING = ("HAVING", "条件")
    AS = ("AS", "别名")
    AND = ("AND", "和")
    OR = ("OR", "或")
    DESC = ("DESC", "降序")
    LIMIT = ("LIMIT", "限制")
    FOR_UPDATE = ("FOR UPDATE", "更新")
    DUPLICATE_UPDATE = ("ON DUPLICATE KEY UPDATE", "重复键更新")
    NULL = ("NULL", "空")
    NOT_NULL = ("NOT NULL", "非空")
    DEFAULT = ("DEFAULT", "默认")
    COMMENT = ("COMMENT", "注释")
    PRIMARY_KEY = ("PRIMARY KEY", "主键")
    UNIQUE = ("UNIQUE", "唯一")
    ENGINE = ("ENGINE", "引擎")
    DEFAULT_CHARSET = ("DEFAULT CHARSET", "默认字符集")
    UNSIGNED = ("UNSIGNED", "无符号")
    ZEROFILL = ("ZEROFILL", "零填充")
    AUTO_INCREMENT = ("AUTO INCREMENT", "自动增长")
    LOCAL = ("LOCAL", "本地")
    INFILE = ("INFILE", "文件")
    REPLACE = ("REPLACE", "替换")
    IGNORE = ("IGNORE", "忽略")
    INTO_TABLE = ("INTO TABLE", "进入表")
    FIELDS_TERMINATED = ("FIELDS TERMINATED BY", "字段终止于")
    ENCLOSED = ("ENCLOSED BY", "包围")
    LINES_TERMINATED = ("LINES TERMINATED BY", "行终止于")
    LINES = ("LINES", "行")



import unittest
from datetime import date, time, datetime

from mag_db.model.sql_type import SqlType


class TestSqlType(unittest.TestCase):

    def test_of_class(self):
        self.assertEqual(SqlType.of_class(int), SqlType.INT)
        self.assertEqual(SqlType.of_class(float), SqlType.FLOAT)
        self.assertEqual(SqlType.of_class(bool), SqlType.BOOLEAN)
        self.assertEqual(SqlType.of_class(str, 100), SqlType.CHAR)
        self.assertEqual(SqlType.of_class(str, 5000), SqlType.VARCHAR)
        self.assertEqual(SqlType.of_class(str, 60000), SqlType.TEXT)
        self.assertEqual(SqlType.of_class(str, 10000000), SqlType.MEDIUMTEXT)
        self.assertEqual(SqlType.of_class(str, 20000000), SqlType.LONGTEXT)
        self.assertEqual(SqlType.of_class(date), SqlType.DATE)
        self.assertEqual(SqlType.of_class(time), SqlType.TIME)
        self.assertEqual(SqlType.of_class(datetime), SqlType.DATETIME)

    def test_get_class(self):
        self.assertEqual(SqlType.get_class("INT"), int)
        self.assertEqual(SqlType.get_class("FLOAT"), float)
        self.assertEqual(SqlType.get_class("BOOLEAN"), bool)
        self.assertEqual(SqlType.get_class("CHAR"), str)
        self.assertEqual(SqlType.get_class("DATE"), date)
        self.assertEqual(SqlType.get_class("TIME"), time)
        self.assertEqual(SqlType.get_class("DATETIME"), datetime)

    def test_str(self):
        self.assertEqual(SqlType.INT.code, "INT")
        self.assertEqual(str(SqlType.FLOAT), "FLOAT")
        self.assertEqual(str(SqlType.DECIMAL), "DECIMAL")
        self.assertEqual(str(SqlType.BOOLEAN), "BOOLEAN")
        self.assertEqual(str(SqlType.CHAR), "CHAR")
        self.assertEqual(str(SqlType.DATE), "DATE")
        self.assertEqual(str(SqlType.TIME), "TIME")
        self.assertEqual(str(SqlType.DATETIME), "DATETIME")

if __name__ == '__main__':
    unittest.main()



import unittest

from mag_db.bean.table import Table
from mag_db.utils.column_utils import ColumnUtils


class TestColumnUtils(unittest.TestCase):
    def test_convert_column_name(self):
        sql = "INSERT INTO table_name1 AS t (t.column1 AS c1, t.column2 AS c2 ) VALUES ('value1', 'value2');"
        table_name = "table_name1"
        table_alias = "t"

        converted_sql = ColumnUtils.convert_column_name(sql, [Table(table_name, alias=table_alias)])
        print(converted_sql)

if __name__ == '__main__':
    unittest.main()



import unittest

from mag_db.bean.table import Table
from parser.sql_parser import SqlParser
from utils.column_utils import ColumnUtils


class TestSqlParser(unittest.TestCase):

    def test_get_tables(self):
        sql = "SELECT * FROM test_table"
        expected_tables = ["test_table"]
        self.assertEqual([table.name for table in SqlParser(sql).tables], expected_tables)

        sql = "INSERT INTO test_table (col1, col2) VALUES (1, 2)"
        expected_tables = ["test_table"]
        self.assertEqual([table.name for table in SqlParser(sql).tables], expected_tables)

        sql = "UPDATE test_table SET col1 = 1 WHERE col2 = 2"
        expected_tables = ["test_table"]
        self.assertEqual([table.name for table in SqlParser(sql).tables], expected_tables)

        sql = "DELETE FROM test_table WHERE col1 = 1"
        expected_tables = ["test_table"]
        self.assertEqual([table.name for table in SqlParser(sql).tables], expected_tables)

    def test_get_column_names(self):
        sql = "SELECT col1, col2 FROM test_table"
        expected_columns = ["col1", "col2"]
        self.assertEqual(SqlParser(sql).column_names, expected_columns)

        sql = "INSERT INTO test_table (col1, col2) VALUES (1, 2)"
        expected_columns = ["col1", "col2"]
        self.assertEqual(SqlParser(sql).column_names, expected_columns)

        sql = "UPDATE test_table SET col1 = 1, col2 = 2 WHERE col3 = 3"
        expected_columns = ["col1", "col2"]
        self.assertEqual(SqlParser(sql).column_names, expected_columns)

    def test_convert_column_name(self):
        sql = "SELECT t.col1, t.col2 FROM test_table AS t"
        tables = [Table(name="test_table", alias="t")]
        expected_sql = "SELECT test_table.col1, test_table.col2 FROM test_table AS t"
        self.assertEqual(ColumnUtils.convert_column_name(sql, tables), expected_sql)

if __name__ == '__main__':
    unittest.main()



from mag_db.handler.dict_type_handler import DictTypeHandler
from mag_db.handler.list_type_handler import ListTypeHandler
from mag_db.handler.decimal_type_handler import DecimalTypeHandler
from mag_db.handler.tuple_type_handler import TupleTypeHandler
from mag_db.handler.bool_type_handler import BoolTypeHandler
from mag_db.handler.bytes_type_handler import BytesTypeHandler
from mag_db.handler.date_type_handler import DateTypeHandler
from mag_db.handler.datetime_type_handler import DatetimeTypeHandler
from mag_db.handler.float_type_handler import FloatTypeHandler
from mag_db.handler.int_type_handler import IntTypeHandler
from mag_db.handler.string_type_handler import StringTypeHandler
from mag_db.handler.time_type_handler import TimeTypeHandler


class TypeConstant:
    # TEXT
    STRING = StringTypeHandler()
    # 时间
    DATETIME = DatetimeTypeHandler()
    DATE = DateTypeHandler()
    TIME = TimeTypeHandler()
    # 数值
    INT = IntTypeHandler()
    FLOAT = FloatTypeHandler()
    DECIMAL = DecimalTypeHandler()
    # 二进制
    BYTES = BytesTypeHandler()
    # 其它
    BOOL = BoolTypeHandler()
    TUPLE = TupleTypeHandler()
    LIST = ListTypeHandler()
    DICT = DictTypeHandler()




from decimal import Decimal

from mag_db.handler.type_handler import T, TypeHandler


class DecimalTypeHandler(TypeHandler):
    def set_parameter(self, parameter: T) -> Decimal | None:
        return parameter


from typing import Any, Dict, List, Optional

from mag_db.bean.foreign_key import ForeignKey
from mag_db.bean.primary_key import PrimaryKey
from mag_db.bean.column import Column
from mag_db.bean.table import Table


class TableInfo:
    def __init__(self, tables: List[Table], columns: List[Column] = [], primary_key: Optional[PrimaryKey] = None,
                 foreign_keys: List[ForeignKey] = [], distinct: bool = False, values: Optional[List[Any]] = None,
                 where : Optional[str] = None, group_by:Optional[str] = None, having: Optional[str] = None,
                 order_by:Optional[str]= None, charset: Optional[str] = None, engine: Optional[str] = None,
                 comment: Optional[str] = None, column_name_map: Optional[Dict[str, str]] = None, auto_increment: bool= False) -> None:
        self.tables = tables
        self.columns = columns
        self.__primary_key = primary_key
        self.foreign_keys = foreign_keys
        self.values = values
        self.distinct = distinct
        self.where = where
        self.group_by = group_by
        self.having = having
        self.order_by = order_by
        self.charset = charset
        self.engine = engine
        self.comment = comment
        self.auto_increment = auto_increment

        self.column_name_map = column_name_map
        if not self.column_name_map:
            self.column_name_map = TableInfo.__get_column_name_map(columns)

    @property
    def table_names(self) -> List[str]:
        return [table.name for table in self.tables]

    def get_column_names(self, is_query: bool = False) -> List[str]:
        column_names = [column.name for column in self.columns]

        # 对于非查询类操作，如主键是自增型，则排除
        if not is_query and self.auto_increment and self.__primary_key:
            column_names.remove(self.__primary_key.default_column)

        return column_names

    @property
    def default_primary_key(self) -> Optional[str]:
        return self.__primary_key.default_column if self.__primary_key else None

    def __str__(self):
        foreign_keys_str_list = [foreign_key.__str__() for foreign_key in self.foreign_keys]

        parts = [ f"tables=[{', '.join(self.table_names)}]",
                  f"columns=[{', '.join(self.get_column_names())}]" if self.columns else "",
                  f"primary_key={self.__primary_key}" if self.__primary_key else "",
                  f"foreign_keys=[{', '.join(foreign_keys_str_list)}]" if len(self.foreign_keys) > 0 else "",
                  f"distinct={self.distinct}",
                  f"values={self.values}" if self.values else "",
                  f"where={self.where}" if self.where else "",
                  f"group_by={self.group_by}" if self.group_by else "",
                  f"having={self.having}" if self.having else "",
                  f"order_by={self.order_by}" if self.order_by else "",
                  f"column_name_map={self.column_name_map}" if self.column_name_map else ""
        ]
        # 过滤掉空字符串
        parts = [part for part in parts if part]
        return f"TableInfo({', '.join(parts)})"

    @classmethod
    def __get_column_name_map(cls, columns: List[Column]) -> Dict[str, str]:
        mappings = {}
        for column in columns:
            map_value = column.mapping
            if map_value:
                mappings[column.name] = map_value
        return mappings



from typing import List, Optional

import sqlparse

from mag_db.bean.table_info import TableInfo
from mag_db.parser.create_parser import CreateParser
from mag_db.parser.delete_parser import DeleteParser
from mag_db.parser.drop_parser import DropParser
from mag_db.parser.insert_parser import InsertParser
from mag_db.parser.query_parser import QueryParser
from mag_db.parser.update_parser import UpdateParser


class SqlParser:
    def __init__(self, sql: str):
        parser = None

        if sql:
            stmt = sqlparse.parse(sql)[0]

            if stmt.get_type() == 'SELECT':
                parser = QueryParser(stmt)
            elif stmt.get_type() == 'UPDATE':
                parser = UpdateParser(stmt)
            elif stmt.get_type() == 'INSERT':
                parser = InsertParser(stmt)
            elif stmt.get_type() == 'DELETE':
                parser = DeleteParser(stmt)
            elif stmt.get_type() == 'DROP':
                parser = DropParser(stmt)
            elif stmt.get_type() == 'CREATE':
                parser = CreateParser(stmt)

        self.sql = sql
        self.__table_info = parser.table_info if parser else TableInfo([])

    @property
    def tables(self):
        return self.__table_info.tables

    @property
    def table_names(self) -> List[str]:
        return self.__table_info.table_names

    def column_names(self, is_query: bool = False) -> List[str]:
        return self.__table_info.get_column_names(is_query)

    @property
    def default_primary_key(self) -> Optional[str]:
        return self.__table_info.default_primary_key



from mag_db.builder.base_sql import BaseSql
from mag_db.model.operator_type import OperatorType


class Drop(BaseSql):
    """
    Drop语句操作类

    @description: 用于构建DROP语句。
    @version: v1.0
    @date: 2019/8/30
    """
    def __init__(self, table_name: str):
        """
        构造方法

        :param table_name: 表名
        """
        super().__init__()

        self.append_operator(OperatorType.DROP_TABLE).append_string(table_name)


from enum import Enum
from inspect import get_annotations
from typing import Dict, List, Optional, Type, TypeVar

from mag_tools.model.base_enum import BaseEnum
from mag_tools.utils.common.class_utils import ClassUtils
from mag_tools.utils.data.string_utils import StringUtils

from mag_db.bean.column import Column

K = TypeVar('K')
V = TypeVar('V')

class ColumnNamesMapping:
    """
    列名信息映射表类
    提供列名信息映射相关的操作。列名格式：“表名.列名” 或 ”列名”，列信息包括列名、类型处理器和映射目标名。
    """
    def __init__(self):
        self.column_infos: Dict[str, str] = {}
        self.column_types: Dict[str, Type] = {}

    def put(self, column_name: str, cls_type: Type, target_name: str):
        """
        放入列名映射
        :param column_name: 列名，可含表名的完整格式，也可不含表名或别名
        :param cls_type: 映射的数据类型
        :param target_name: 映射的目标名
        """
        column_name = StringUtils.pick_head(column_name, " AS ")
        self.column_infos[column_name] = target_name
        self.column_types[column_name] = cls_type

    def get_target_name(self, column_name: str) -> str:
        """
        获取指定列名映射的目标名
        :param column_name: 列名，可含表名和别名的完整格式，也可不含表名或别名
        :return: 映射的目标名
        """
        match_column_name = self.__find_match(list(self.column_infos.keys()), column_name)
        return self.column_infos.get(match_column_name) if match_column_name else None

    def get_class_type(self, column_name: str) -> Type:
        match_column_name = self.__find_match(list(self.column_types.keys()), column_name)
        return self.column_types.get(match_column_name) if match_column_name else None

    @classmethod
    def get_by_class(cls, bean_class: Type, column_names: List[str], column_name_map: Dict[str, str] = None):
        if column_name_map is None:
            column_name_map = {}

        # 缺省地，将Bean的字段名作为数据库列名的映射，然后与设定的映射表合并
        total_map = {column_name: column_name for column_name in column_names}

        # 将旧列名和字段映射表中未包含的列名添加到新表
        for column_name, map_name in column_name_map.items():
            if column_name not in total_map:
                total_map[column_name] = map_name

        return cls.__get_by_bean(total_map, bean_class)

    @classmethod
    def get_by_field_map(cls, field_map: Dict[K, V], column_names: List[str], column_name_map: Optional[Dict[str, str]] = None):
        if column_name_map is None:
            column_name_map = {}

        # 缺省地，将Bean的字段名作为数据库列名的映射，然后与设定的映射表合并
        total_map = {column_name: column_name for column_name in column_names}

        # 将旧列名和字段映射表中未包含的列名添加到新表
        for column_name, map_name in column_name_map.items():
            if column_name not in total_map:
                total_map[column_name] = map_name

        return cls.__get_by_field_map(total_map, field_map)

    @classmethod
    def __get_by_bean(cls, column_names_map: Dict[str, str], bean_class: Type):
        mapping = cls()

        for column_name in column_names_map.keys():
            bean_field_name = Column.to_field_name(column_names_map.get(column_name))
            target_class = get_annotations(bean_class).get(bean_field_name)
            target_class = ClassUtils.get_origin_type(target_class)

            if target_class and isinstance(target_class, type) and issubclass(target_class, Enum):
                target_class = type(target_class.code) if issubclass(target_class, BaseEnum) else target_class.name

            mapping.put(column_name, target_class, bean_field_name)

        return mapping

    @classmethod
    def __get_by_field_map(cls, column_names_map: Dict[str, str], field_map: Dict[K, V]):
        mapping = cls()

        for column_name in column_names_map.keys():
            target_name_type = type(field_map.get(column_name))
            if target_name_type:
                mapping.put(column_name, target_name_type, column_name)

        return mapping

    @classmethod
    def __find_match(cls, column_names: List[str], column_name: str) -> str:
        return next((name for name in column_names if name == column_name or name == StringUtils.pick_tail(column_name, ".")), None)



from typing import List

from mag_tools.log.logger import Logger
from mag_tools.model.log_type import LogType

from mag_db.core.db_session import DBSession
from mag_db.core.transaction_cache import TransactionCache
from mag_db.sql.base_executor import BaseExecutor


class IndexInsertExecutor(BaseExecutor):
    def __init__(self, sql: str, session: DBSession):
        super().__init__(sql, session)

    def execute(self)->List[int]:
        self.check()

        tx = None
        try:
            values = self.prepare()

            tx = TransactionCache.get_current_tx(self._session.datasource_name)
            connection = self.get_connection(tx)
            with connection.cursor() as cursor:
                insert_ids = []
                if values:
                    cursor.executemany(self._sql, values)

                    last_id = cursor.lastrowid
                    insert_num = cursor.rowcount
                    for i in range(insert_num):
                        insert_ids.append(last_id + i)
                else:
                    cursor.execute(self._sql)
                    insert_ids.append(cursor.lastrowid)

                if tx is None:
                    connection.commit()

                return insert_ids
        except (NotImplementedError, Exception):
            Logger.throw(LogType.DAO,f"执行SQL失败: {self._sql}")


from typing import Any, Dict, List, get_args

from mag_tools.utils.common.class_utils import ClassUtils
from mag_tools.jsonparser.json_parser import JsonParser
from typing_extensions import get_origin

from mag_db.data.db_output import DbOutput


class ResultSetHelper:
    @staticmethod
    def to_maps(result_set: list[Dict[str, Any]], db_output: DbOutput) -> List[Dict[str, Any]]:
        return [
            {
                db_output.column_name_mapping.get_target_name(column_name) or column_name:
                    row_map[column_name] for column_name in db_output.column_names
            }
            for row_map in result_set
        ]

    @staticmethod
    def to_beans(result_set: list[Dict[str, Any]], db_output: DbOutput) -> List[Any]:
        result_list = []
        for row_map in result_set:
            kwargs = {}
            for column_name in db_output.column_names:
                target_name = db_output.get_target_name(column_name)
                target_type = ClassUtils.get_field_type_of_class(db_output.result_class, target_name)
                target_type = ClassUtils.get_origin_type(target_type)
                value = row_map.get(column_name)

                if get_origin(target_type) is tuple:
                    args = get_args(target_type)
                    if args:
                        kwargs[target_name] = JsonParser.to_tuple(value, args[0])
                elif get_origin(target_type) is list:
                    args = get_args(target_type)
                    if args:
                        kwargs[target_name] = JsonParser.to_list(value, args[0])
                elif get_origin(target_type) is dict:
                    args = get_args(target_type)
                    if args:
                        kwargs[target_name] = JsonParser.to_map(value, str, args[0])
                else:
                    kwargs[target_name] = value

            result_list.append(db_output.result_class(**kwargs))

        return result_list



import unittest

from mag_db.bean.column import Column

class TestColumn(unittest.TestCase):

    def test_constructor(self):
        column = Column.of_whole_name("test_table.test_column AS test_alias")
        self.assertEqual(column.table_name, "test_table")
        self.assertEqual(column.name, "test_column")
        self.assertEqual(column.alias, "test_alias")

    def test_get_alias(self):
        column = Column(table_name="test_table", name="test_column")
        self.assertEqual(column.alias, "test_table__test_column")

    def test_get_whole_name(self):
        column = Column(table_name="test_table", name="test_column", alias="test_alias")
        self.assertEqual(column.get_whole_name(), "test_table.test_column AS test_alias")

    def test_get_short_name(self):
        column = Column(table_name="test_table", name="test_column")
        self.assertEqual(column.short_name, "test_table.test_column")

    def test_is_function(self):
        column = Column(name="SUM(test_column)")
        self.assertTrue(column.is_function)

    def test_is_empty(self):
        column = Column(name="")
        self.assertTrue(column.is_empty)

    def test_get_comment(self):
        column = Column(_comment="This is a comment with 'special' characters\".")
        self.assertEqual(column.comment, "This is a comment with special characters.")

    def test_is_unique(self):
        column = Column(uniques=["unique1", "unique2"])
        self.assertTrue(column.is_unique)

if __name__ == '__main__':
    unittest.main()



from decimal import Decimal
from enum import Enum
from typing import Type, Optional
from datetime import date, time, datetime

from mag_tools.model.base_enum import BaseEnum


class SqlType(BaseEnum):
    # 数值
    TINYINT = ("TINYINT", int, "小整数")
    SMALLINT = ("SMALLINT", int, "短整数")
    MEDIUMINT = ("MEDIUMINT", int, "中整数")
    INT = ("INT", int, "整数")
    BIGINT = ("BIGINT", int, "长整数")
    TINYINT_UNSIGNED = ("TINYINT UNSIGNED", int, "无符号小整数")
    SMALLINT_UNSIGNED = ("SMALLINT UNSIGNED", int, "无符号短整数")
    MEDIUMINT_UNSIGNED = ("MEDIUMINT UNSIGNED", int, "无符号中整数")
    INT_UNSIGNED = ("INT UNSIGNED", int, "无符号整数")
    BIGINT_UNSIGNED = ("BIGINT UNSIGNED", int, "无符号长整数")
    FLOAT = ("FLOAT", float, "单精度浮点数")
    DOUBLE = ("DOUBLE", float, "双精度浮点数")
    DOUBLE_PRECISION = ("DOUBLE PRECISION", float, "双精度浮点数")
    REAL = ("REAL", float, "浮点数")
    DECIMAL = ("DECIMAL", Decimal, "大数")
    DECIMAL_UNSIGNED = ("DECIMAL UNSIGNED", Decimal, "无符号大数")
    DEC = ("DEC", Decimal, "大数")
    NUMERIC = ("NUMERIC", Decimal, "大数")
    SERIAL = ("SERIAL", int, "自增整数")
    INT4 = ("INT4", int, "整数")
    # 二进制
    BIT = ("BIT", bytes, "位")
    FIXED = ("FIXED", Decimal, "定点数数据")
    BINARY = ("BINARY", bytes, "二进制数据")
    VARBINARY = ("VARBINARY", bytes, "可变二进制数据")
    TINYBLOB = ("TINYBLOB", bytes, "TINY BLOB")
    MEDIUMBLOB = ("MEDIUMBLOB", bytes, "MEDIUM BLOB")
    BLOB = ("BLOB", bytes, "BLOB")
    LONGBLOB = ("LONGBLOB", bytes, "LONG BLOB")
    RAW = ("RAW", bytes, "二进制数据")
    # Char
    CHAR = ("CHAR", str, "字符串")
    VARCHAR = ("VARCHAR", str, "可变字符串")
    NCHAR = ("NCHAR", str, "字符串")
    VARCHAR2 = ("VARCHAR2", str, "可变字符串")
    NVARCHAR = ("NVARCHAR", str, "可变字符串")
    NVARCHAR_2 = ("NVARCHAR2", str, "可变字符串")
    CHARACTER_VARYING = ("CHARACTER VARYING", str, "可变字符串")
    # Text
    TINYTEXT = ("TINYTEXT", str, "短文本")
    TEXT = ("TEXT", str, "文本")
    MEDIUMTEXT = ("MEDIUMTEXT", str, "中长文本")
    LONGTEXT = ("LONGTEXT", str, "长文本")
    JSON = ("JSON", str, "Json文本")
    NTEXT = ("NTEXT", str, "文本")
    JSONB = ("JSONB", str, "Json文本")
    # 日期时间
    DATE = ("DATE", date, "日期")
    TIME = ("TIME", time, "时间")
    TIMESTAMP = ("TIMESTAMP", datetime, "时间戳")
    DATETIME = ("DATETIME", datetime, "日期时间")
    YEAR = ("YEAR", str, "年份")
    DATETIME_2 = ("DATETIME2", datetime, "日期时间")
    SMALLDATETIME = ("SMALLDATETIME", datetime, "日期时间")
    DATETIMEOFFSET = ("DATETIMEOFFSET", datetime, "日期时间")
    # 几何图形
    GEOMETRY = ("GEOMETRY", object, "几何图形")
    POINT = ("POINT", object, "点")
    LINESTRING = ("LINESTRING", object, "线段")
    POLYGON = ("POLYGON", object, "多边形")
    MULTIPOINT = ("MULTIPOINT", object, "多点")
    MULTILINESTRING = ("MULTILINESTRING", object, "多线段")
    MULTIPOLYGON = ("MULTIPOLYGON", object, "多个多边形")
    GEOMETRYCOLLECTION = ("GEOMETRYCOLLECTION", object, "图形集")
    # 其它
    BOOLEAN = ("BOOLEAN", bool, "布尔值")
    BOOL = ("BOOL", bool, "布尔值")
    ENUM = ("ENUM", Enum, "枚举")
    SET = ("SET", str, "集合")
    MONEY = ("MONEY", Decimal, "货币")
    OBJECT = ("OBJECT", object, "对象")
    IMAGE = ("IMAGE", bytes, "图像")
    CIDR = ("CIDR", str, "IP地址")
    INET = ("INET", str, "IP地址")
    MACADDR = ("MACADDR", str, "MAC地址")
    MACADDR8 = ("MACADDR8", str, "MAC地址")

    def __init__(self, code: str, cls: Type, desc: str):
        super().__init__(code, desc)
        self.__cls = cls

    @classmethod
    def of_class(cls, class_type: Type, size: Optional[int] = None) -> 'SqlType':
        """
        根据列的JDK类型获取对应的枚举值
        :param class_type: 列的JDK类型
        :param size: 列长度
        :return: 对应的枚举值
        """
        if class_type == int:
            return cls.INT
        elif class_type == float:
            return cls.FLOAT
        elif class_type == Decimal:
            return cls.DECIMAL
        elif class_type == bool:
            return cls.BOOLEAN
        elif class_type == str:
            if size is None or size < 127:
                return cls.CHAR
            elif size < 8 * 1024:
                return cls.VARCHAR
            elif size < 64 * 1024:
                return cls.TEXT
            elif size < 16 * 1024 * 1024:
                return cls.MEDIUMTEXT
            else:
                return cls.LONGTEXT
        elif class_type == date:
            return cls.DATE
        elif class_type == time:
            return cls.TIME
        elif class_type == datetime:
            return cls.DATETIME
        else:
            for item in cls:
                if item.__cls == class_type:
                    return item
        raise ValueError(f"{class_type} is not a valid class type for {cls.__name__}")

    @classmethod
    def get_class(cls, code: str) -> Type:
        """
        根据列类型名获取对应的JDK类
        :param code: 列类型名
        :return: 对应的JDK类
        """
        return cls.of_code(code).__cls


import re
from typing import List, Optional, Type

from mag_tools.exception.dao_exception import DaoException
from mag_tools.model.symbol import Symbol

from bean.column import Column
from mag_db.bean.table import Table
from mag_db.model.operator_type import OperatorType
from model.sql_type import SqlType
from utils.column_utils import ColumnUtils


class TableUtils:
    @staticmethod
    def get_table_from_sql(create_sql: str):
        table = Table()

        sql = create_sql.replace(OperatorType.CREATE_TABLE.code, "").strip()

        idx = sql.find(Symbol.OPEN_PAREN.code)
        if idx <= 0:
            raise DaoException(f"建表SQL语句有错：{create_sql}")

        table.names = [sql[:idx].strip()]

        sql = sql[idx + 1:]
        idx = sql.rfind(Symbol.CLOSE_PAREN.code)
        if idx <= 0:
            raise DaoException(f"建表SQL语句有错：{create_sql}")

        sql = sql[:idx]

        idx = sql.find(OperatorType.PRIMARY_KEY.code)
        if idx > 0:
            table.primary_key = sql[idx:].replace(Symbol.OPEN_PAREN.code, "").replace(Symbol.CLOSE_PAREN.code, "")
            sql = sql[:idx].strip()

        column_infos = re.split(r',\s*(?![^()]*\))', sql)
        columns = TableUtils.to_column(column_infos)

        table.set_columns(columns)

        return table

    @staticmethod
    def to_column(column_infos: List[str]):
        columns = []

        for column_info in column_infos:
            column = Column()

            column_info = column_info.strip()

            idx = column_info.find(Symbol.BLANK.code)
            if idx <= 0:
                raise DaoException("列描述语句错误")

            name = column_info[:idx]
            column.name = name

            column_info = column_info[idx + 1:].upper()

            idx = column_info.find(Symbol.BLANK.code)
            if idx <= 0:
                raise DaoException("列描述语句错误")

            type_str = column_info[:idx]
            column_info = column_info[idx + 1:]

            idx = type_str.find(Symbol.OPEN_PAREN.code)
            if idx > 0:
                idx_close = type_str.find(Symbol.CLOSE_PAREN.code)
                size = type_str[idx + 1:idx_close]
                column.column_size = int(size)
                type_str = type_str[:idx]

            column.sql_type = SqlType.of_code(type_str)
            column.can_null = OperatorType.NOT_NULL not in column_info
            column.auto_increment = OperatorType.AUTO_INCREMENT.code in column_info

            columns.append(column)

        return columns

    @staticmethod
    def to_table(bean_cls: Type, table_name: str, comment: Optional[str] = None, auto_increment_value: bool = True):
        table = Table()
        table.names = [table_name]

        fields = [field for field in dir(bean_cls) if not callable(getattr(bean_cls, field)) and not field.startswith("__")]
        table.__columns = [ColumnUtils.to_column(field) for field in fields]
        table.comment = comment
        table.auto_increment = auto_increment_value

        return table

    @staticmethod
    def clear_table_name(name):
        return re.sub(r'[^\w]', '', name.strip()) if name else None



from mag_tools.model.base_enum import BaseEnum

class DbType(BaseEnum):
    MYSQL = ("mysql", 3306, "MySQL")
    ORACLE = ("oracle", 1521, "Oracle")
    SQL_SERVER = ("sqlserver", 1433, "SQL Server")
    POSTGRE_SQL = ("postgresql", 5432, "PostgreSQL")
    DM = ("dm", 5236, "DM")
    DB2 = ("db2", 5000, "DB2")

    def __init__(self, code:str, port:int, desc:str):
        super().__init__(code, desc)
        self.__port = port

    @property
    def port(self):
        return self.__port

if __name__ == '__main__':
    # 示例用法
    _db_type = DbType.of_code("mysql")
    print(f"Code: {_db_type.code}, Desc: {_db_type.desc}, Port: {_db_type.port}, Display Name: {_db_type.desc}")



from mag_tools.model.base_enum import BaseEnum


class EngineType(BaseEnum):
    """
    数据库引擎类型枚举类

    @author: Xiaolong Cao
    @version: v1.2
    @description: 数据库引擎类型包括：MyISAM、InnoDB、MEMORY、MRG_MYISAM、ARCHIVE、CSV、BLACKHOLE、FEDERATED、NDB
    @date: 2017/9/21
    """
    INNO_DB = ("InnoDB", "支持事务处理和外键约束")
    MY_ISAM = ("MyISAM", "不支持事务处理和外键约束，适用于只读数据")
    MEMORY = ("MEMORY", "将数据存储在内存中，适用于临时数据")
    MRG_MYISAM = ("MRG_MYISAM", "将多个MyISAM表合并为一个表")
    ARCHIVE = ("ARCHIVE", "用于存储大量的归档数据")
    CSV = ("CSV", "将数据存储为CSV格式文件")
    BLACKHOLE = ("BLACKHOLE", "不存储数据，仅用于复制")
    FEDERATED = ("FEDERATED", "访问远程数据库中的表")
    NDB = ("NDB", "用于MySQL Cluster，支持高可用性和分布式存储")


from typing import Any

from mag_tools.utils.data.date_utils import DateUtils

from mag_db.handler.type_handler import T, TypeHandler


class DatetimeTypeHandler(TypeHandler):
    def set_parameter(self, parameter: T) -> Any | None:
        return DateUtils.to_string(parameter, '%Y-%m-%d %H:%M:%S')



from mag_tools.model.base_enum import BaseEnum

from mag_db.model.sql_type import SqlType


class PostgresConvertType(BaseEnum):
    """
    SqlServer的列数据类型-SqlType映射的枚举类

    @description: 建立列类型、对应的类名的映射关系
    @version: v1.2
    @date: 2017/7/22
    """
    # 字符与文本
    CHARACTER = ("CHARACTER", SqlType.CHAR, "字符串")
    CHAR = ("CHAR", SqlType.CHAR, "字符串")
    CHARACTER_VARYING = ("CHARACTER VARYING", SqlType.VARCHAR, "可变字符串")
    VARCHAR = ("VARCHAR", SqlType.VARCHAR, "可变字符串")
    TEXT = ("TEXT", SqlType.TEXT, "文本")
    # 其它
    BYTEA = ("BYTEA", SqlType.BINARY, "二进制数")
    BOOLEAN = ("BOOLEAN", SqlType.BOOLEAN, "布尔值")
    BOOL = ("BOOL", SqlType.BOOLEAN, "布尔值")
    ENUM = ("ENUM", SqlType.ENUM, "枚举")
    CIDR = ("CIDR", SqlType.VARCHAR, "IP地址")
    INET = ("INET", SqlType.VARCHAR, "IP地址")
    MACADDR = ("MACADDR", SqlType.VARCHAR, "MAC地址")
    MACADDR8 = ("MACADDR8", SqlType.VARCHAR, "MAC地址")
    JSON = ("JSON", SqlType.JSON, "JSON文本")
    JSONB = ("JSONB", SqlType.JSON, "JSON文本")
    # 日期和时间
    DATE = ("DATE", SqlType.DATE, "日期")
    TIME = ("TIME", SqlType.TIME, "时间")
    TIMESTAMP = ("TIMESTAMP", SqlType.TIMESTAMP, "时间戳")
    INTERVAL = ("INTERVAL", SqlType.BIGINT, "时间间隔")
    # 数值
    SMALLINT = ("SMALLINT", SqlType.SMALLINT, "短整数")
    INTEGER = ("INTEGER", SqlType.MEDIUMINT, "整数")
    INT4 = ("INT4", SqlType.INT, "整数")
    BIGINT = ("BIGINT", SqlType.BIGINT, "长整数")
    DECIMAL = ("DECIMAL", SqlType.DECIMAL, "大数")
    NUMERIC = ("NUMERIC", SqlType.NUMERIC, "大数")
    REAL = ("REAL", SqlType.FLOAT, "浮点数")
    DOUBLE_PRECISION = ("DOUBLE PRECISION", SqlType.DOUBLE, "双精度浮点数")
    SERIAL = ("SERIAL", SqlType.SERIAL, "自增整数")

    def __init__(self, code: str, mysql: SqlType, desc: str):
        super().__init__(code, desc)
        self.mysql = mysql


import sqlparse
from sqlparse.sql import Statement, Token
from sqlparse.tokens import Punctuation

from mag_db.bean.column import Column
from mag_db.bean.table_info import TableInfo
from mag_db.parser.base_parser import BaseParser
from mag_db.utils.column_utils import ColumnUtils


class QueryParser(BaseParser):
    def _parse(self, stmt: Statement) -> TableInfo:
        # 提取表名、表别名
        tables = super()._find_table(stmt, {'FROM'})

        # 将SQL语句中”表别名.列名“转为”表名.列名“
        sql = ColumnUtils.convert_column_name(stmt.value, tables)
        stmt = sqlparse.parse(sql)[0]

        # 提取列名及其别名
        columns_token = super()._find_keyword(stmt, "SELECT")
        column_names_str = super()._get_value_of_token(stmt, columns_token)
        columns = super()._parse_column_names(column_names_str)

        # 提取条件部分
        where = super()._find_where(stmt)

        # 提取 GROUP BY 子句
        group_by = self.__group_by(stmt)
        having = self.__having(stmt)

        # 提取 ORDER BY 子句
        order_by = self.__order_by(stmt)

        return TableInfo(tables, columns, where=where, group_by=group_by, having=having, order_by=order_by)

    @classmethod
    def __parse_column_names(cls, columns_token: Token):
        columns = []
        for token in columns_token.tokens:
            if token.ttype is None:
                for part_token in token.tokens:
                    if part_token.ttype is None:
                        part_values = part_token.value.split()
                        if len(part_values) == 1:
                            column = Column(name=part_values[0])
                            columns.append(column)
                        elif len(part_values) == 3 and part_values[1].upper() == 'AS':
                            column = Column(name=part_values[0], alias=part_values[2])
                            columns.append(column)
        return columns

    @classmethod
    def __group_by(cls, stmt: Statement) -> [str]:
        # 提取 GROUP BY 子句
        group_by = []
        group_by_token = BaseParser._find_keyword(stmt, 'GROUP BY')
        if group_by_token:
            next_token = BaseParser._next_token(stmt, group_by_token)
            while next_token and not BaseParser._is_keyword(next_token.value):
                if next_token.ttype is not Punctuation:
                    group_by.append(next_token.value)

                next_token = BaseParser._next_token(stmt, next_token)
        return ' '.join(group_by) if group_by else None

    @classmethod
    def __having(cls, stmt: Statement) -> [str]:
        # 提取 HAVING 子句
        having_token = BaseParser._find_keyword(stmt, 'HAVING')
        having = []
        if having_token:
            next_token = BaseParser._next_token(stmt, having_token)
            while next_token and not BaseParser._is_keyword(next_token.value):
                having.append(next_token.value)
                next_token = BaseParser._next_token(stmt, next_token)
        return ' '.join(having) if len(having) > 0 else None

    @classmethod
    def __order_by(cls, stmt: Statement):
        # 提取 ORDER BY 子句
        order_by_token = BaseParser._find_keyword(stmt, 'ORDER BY')
        order_by = []
        if order_by_token:
            next_token = BaseParser._next_token(stmt, order_by_token)
            while next_token and not BaseParser._is_keyword(next_token.value):
                if next_token.ttype is not Punctuation:
                    order_by.append(next_token.value)

                next_token = QueryParser._next_token(stmt, next_token)
        return ' '.join(order_by) if len(order_by) > 0 else None


if __name__ == '__main__':
    # 示例用法
    _sql = "SELECT t.column1 AS c1, t.column2 AS c2 FROM table_name1 AS t WHERE t.id IN (1, 2, 3) AND t.name = 'John' GROUP BY t.column1 HAVING COUNT(t.column1) > 1 ORDER BY t.column2;"
    _stmt = sqlparse.parse(_sql)[0]
    _parser = QueryParser(_stmt)
    if _parser.table_info:
        print(_parser.table_info)



import sqlparse
from mag_tools.exception.dao_exception import DaoException
from mag_tools.utils.common.string_utils import StringUtils
from sqlparse.sql import Comparison, Statement, Token

from mag_db.bean.column import Column
from mag_db.bean.table_info import TableInfo
from mag_db.parser.base_parser import BaseParser
from mag_db.utils.column_utils import ColumnUtils


class UpdateParser(BaseParser):
    def _parse(self, stmt: Statement) -> TableInfo:
        # 提取表名及别名
        tables = super()._find_table(stmt, {'UPDATE'})

        # 将SQL语句中”表别名.列名“转为”表名.列名“
        sql = ColumnUtils.convert_column_name(stmt.value, tables)
        stmt = sqlparse.parse(sql)[0]

        # 提取列名及其数值
        columns_token = self.__find_column_name_token(stmt)
        columns, values = self.__parse_column(columns_token)

        # 提取条件部分
        where = super()._find_where(stmt)

        return TableInfo(tables, columns, values=values, where=where)

    @classmethod
    def __find_column_name_token(cls, stmt: Statement):
        column_name_token = None
        set_token = super()._find_keyword(stmt, "SET")
        if set_token:
            column_name_token = super()._next_token(stmt, set_token)

        if not column_name_token or not isinstance(column_name_token, Token) or column_name_token.ttype is not None:
            raise DaoException(f"SQL解析错误，找不到列名:{stmt.value}")

        return column_name_token

    @classmethod
    def __parse_column(cls, columns_token: Token):
        columns = []
        values = []
        for token in columns_token.tokens:
            if isinstance(token, Comparison):
                column_name = StringUtils.pick_head(token.value, '=')
                column_value = StringUtils.pick_tail(token.value, '=')
                columns.append(Column(column_name))
                values.append(column_value)
        return columns, values

if __name__ == "__main__":
    # 示例用法
    _sql = "UPDATE table_name1 AS t SET t.column1 = 'value1', t.column2 = 'value2' WHERE id = 1;"

    _stmt = sqlparse.parse(_sql)[0]
    _result = UpdateParser(_stmt)
    if _result.table_info:
        print(_result.table_info)



import re
from typing import List, Optional, Tuple

import sqlparse
from mag_tools.exception.dao_exception import DaoException
from mag_tools.utils.common.string_utils import StringUtils
from sqlparse.sql import Statement
from sqlparse.tokens import Token

from mag_db.bean.column import Column
from mag_db.bean.foreign_key import ForeignKey
from mag_db.bean.primary_key import PrimaryKey
from mag_db.bean.table_info import TableInfo
from mag_db.model.sql_type import SqlType
from mag_db.parser.base_parser import BaseParser


class CreateParser(BaseParser):
    def _parse(self, stmt: Statement) -> TableInfo:
        # 提取表名及别名
        tables = super()._find_table(stmt, {'EXISTS', 'TABLE'})

        charset_token = super()._find_keyword(stmt, 'CHARSET')
        charset = super()._get_value_of_token(stmt, charset_token)

        engine_token = super()._find_keyword(stmt, 'ENGINE')
        engine = super()._get_value_of_token(stmt, engine_token)

        comment_token = super()._find_keyword(stmt, 'COMMENT')
        comment = super()._get_value_of_token(stmt, comment_token)

        auto_increment_token = super()._find_keyword(stmt, 'AUTO_INCREMENT')
        auto_increment = super()._get_value_of_token(stmt, auto_increment_token)

        # 提取列名及其数据类型、约束和注释
        columns_token = super()._find_parenthesis(stmt)
        column_items = self.__split_columns_str(columns_token.value)

        columns = []
        primary_key = None
        foreign_keys = []
        for item in column_items:
            item = item.strip()
            if item.startswith('FOREIGN KEY'):
                foreign_key = self.__parse_foreign_key(item)
                foreign_keys.append(foreign_key)
            elif item.startswith('PRIMARY KEY') or item.startswith('CONSTRAINT'):
                primary_key = self.__parse_primary_key(item)
            else:
                column = self.__parse_column(item)
                columns.append(column)
                if column.is_primary_key:
                    primary_key = PrimaryKey([column.name])

        if primary_key and auto_increment:
            primary_key.increment = auto_increment

        return TableInfo(tables, columns, primary_key=primary_key, foreign_keys=foreign_keys, charset=charset,
                         engine=engine, comment=comment)

    @classmethod
    def __parse_column(cls, column_str: str) -> Column:
        column_stmt = sqlparse.parse(column_str)[0]

        column_name_token = column_stmt.tokens[0]
        if not column_name_token or not isinstance(column_name_token, sqlparse.sql.Identifier):
            raise DaoException(f'找不到列名[{column_str}]')
        column_name = column_name_token.value if column_name_token.value else None

        type_token = super()._next_token(column_stmt, column_name_token)

        if not type_token or type_token.ttype not in (None, Token.Name.Builtin):
            raise DaoException(f'找不到列名[{column_str}]的数据类型')
        sql_type, column_size, decimal = cls.__parse_column_type(type_token.value)

        comment_token = super()._find_keyword(column_stmt, 'COMMENT')
        comment = super()._get_value_of_token(column_stmt, comment_token)

        primary_token = super()._find_keyword(column_stmt, 'PRIMARY KEY')
        is_primary_key = primary_token is not None

        auto_increment_token = super()._find_keyword(column_stmt, 'AUTO_INCREMENT')
        is_auto_increment = auto_increment_token is not None

        return Column(column_name, sql_type=sql_type, column_size=column_size, decimal_digits = decimal, _comment=comment,
                      is_primary_key=is_primary_key, is_auto_increment=is_auto_increment)

    @classmethod
    def __parse_foreign_key(cls, foreign_key_str: str) -> Optional[ForeignKey]:
        if foreign_key_str is None:
            return None

        foreign_key_stmt = sqlparse.parse(foreign_key_str)[0]
        column_names_token = super()._find_keyword(foreign_key_stmt, 'KEY')
        column_names_values = super()._get_value_of_token(foreign_key_stmt, column_names_token)

        if not column_names_values:
            raise DaoException(f'外键语句[{foreign_key_str}]中找不到列名')

        column_names_values = column_names_values.strip('()')
        column_names = [col.strip() for col in column_names_values.split(',')] if column_names_values else None

        references_token = super()._find_keyword(foreign_key_stmt, 'REFERENCES')
        references_value = super()._get_value_of_token(foreign_key_stmt, references_token)
        references_table, references_columns_str, _ = StringUtils.split_by_keyword(references_value, '()')
        references_columns = references_columns_str.split(',') if references_columns_str else None

        return ForeignKey(column_names, references_table, references_columns)

    @classmethod
    def __parse_primary_key(cls, primary_key_str: str) -> Optional[PrimaryKey]:
        if primary_key_str is None:
            return None

        primary_key_stmt = sqlparse.parse(primary_key_str)[0]
        pk_name_token = super()._find_keyword(primary_key_stmt, 'CONSTRAINT')
        pk_name = super()._get_value_of_token(primary_key_stmt, pk_name_token)

        column_names_token = super()._find_keyword(primary_key_stmt, 'PRIMARY KEY')
        column_names_values = super()._get_value_of_token(primary_key_stmt, column_names_token)

        if not column_names_values:
            raise DaoException(f'主键语句[{primary_key_str}]中找不到列名')

        column_names_values = column_names_values.strip('()')
        column_names = [col.strip() for col in column_names_values.split(',')] if column_names_values else None

        key_seq = [index for index, column_name in enumerate(column_names)] if column_names else None

        key_seq_token = super()._find_keyword(primary_key_stmt, 'KEY_SEQ')
        if key_seq_token:
            key_seq_values = super()._get_value_of_token(primary_key_stmt, key_seq_token)
            key_seq_values = key_seq_values.strip('()')
            key_seq = [key.strip() for key in key_seq_values.split(',')] if key_seq_values else None

        return PrimaryKey(column_names, key_seq, pk_name)

    @classmethod
    def __parse_column_type(cls, type_str: str) -> Tuple[Optional[SqlType], Optional[int], Optional[int]]:
        """
        解析SQL列描述字符串，提取数据类型、长度和小数秒精度

        参数:
        description (str): 列描述字符串

        返回:
        包含数据类型、长度和小数秒精度的元组
        """
        pattern = r"(?P<data_type>\w+)(\((?P<length>\d+)(, (?P<decimal>\d+))?\))?"
        match = re.search(pattern, type_str.strip())
        if not match:
            raise ValueError("Invalid column description format")

        column_info = match.groupdict()
        sql_type = SqlType.of_code(column_info.get('data_type'))
        length = column_info.get('length')
        decimal = column_info.get('decimal')
        return sql_type, int(length) if length else None, int(decimal) if decimal else None

    @classmethod
    def __split_columns_str(cls, columns_str: str) -> List[str]:
        pattern = r"('[^']*'|\"[^\"]*\"|\([^)]*\))"

        columns_str = columns_str.strip().replace('\n', '')
        columns_str = columns_str[1:-1] if len(columns_str) > 1 else columns_str
        columns_str = re.sub(pattern, lambda m: m.group().replace(',', '%%'), columns_str)
        return [column_str.replace('%%', ',') for column_str in columns_str.split(",")]

if __name__ == "__main__":
    _sql = '''
    CREATE TABLE IF NOT EXISTS table_name1 (
        id INT  COMMENT '主键ID', 
        name VARCHAR(50) NOT NULL COMMENT '名称',
        length1 DECIMAL(10,2) NULL COMMENT '长度,缺省值为0',
        created_at TIMESTAMP(6) DEFAULT CURRENT_TIMESTAMP COMMENT "创建时间,缺省为当天", 
        FOREIGN KEY (id,name) REFERENCES other_table(id,name),
        CONSTRAINT pk_name PRIMARY KEY (id, name) 
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='示例表';
    '''

    _stmt = sqlparse.parse(_sql)[0]
    parser = CreateParser(_stmt)
    if parser.table_info:
        print(parser.table_info)




import pymysql
from mag_tools.exception.dao_exception import DaoException
from mag_tools.log.logger import Logger
from mag_tools.model.log_type import LogType

from mag_db.bean.datasource import Datasource
from mag_db.core.transaction import Transaction
from mag_db.core.transaction_cache import TransactionCache
from mag_db.model.db_type import DbType


class DBSession:
    def __init__(self, datasource: Datasource):
        self._datasource = datasource

    @property
    def datasource_name(self):
        return self._datasource.name

    @property
    def max_insert(self):
        return self._datasource.max_insert

    def connect(self):
        connection = None
        try:
            if self._datasource.db_type == DbType.MYSQL:
                connection = pymysql.connect(
                    host=self._datasource.host,
                    port=self._datasource.port,
                    user=self._datasource.username,
                    password=self._datasource.password,
                    database=self._datasource.db_name
                )
            elif self._datasource.db_type == DbType.POSTGRE_SQL:
                    connection = None
                    # psycopg2.connect(
                    # host=self.__datasource_info.server_addr,
                    # user=self.__datasource_info.username,
                    # password=self.__datasource_info.password,
                    # dbname=self.__datasource_info.db_name
                    # )
            elif self._datasource.db_type == DbType.SQL_SERVER:
                    connection = None
                # pyodbc.connect(
                #     f"DRIVER={{SQL Server}};SERVER={self.__datasource_info.server_addr};"
                #     f"DATABASE={self.__datasource_info.db_name};UID={self.__datasource_info.username};"
                #     f"PWD={self.__datasource_info.password}"
                # )
            else:
                raise f"不支持的数据库类型: {self._datasource.db_type}"
        except Exception as err:
            Logger.error(LogType.DAO, f"数据库连接失败: {err}")

        return connection

    @classmethod
    def close(cls, connection):
        if connection:
            connection.close()
            Logger.debug("数据库连接已关闭")

    def begin_transaction(self, name: str):
        try:
            last_tx = TransactionCache.get_current_tx(self._datasource.name)
            if last_tx is None:
                connection = self.connect()
                trans = Transaction(name, self.datasource_name, connection)
                trans.begin_with_connection()
                return trans
            else:
                trans = Transaction.from_parent(last_tx, name)
                trans.begin_without_connection()
                return trans
        except DaoException:
            Logger.throw(LogType.DAO, f"开启事务失败：{name}")

    def begin_new_transaction(self, name: str):
        try:
            connection = self.connect()
            trans = Transaction(name, self.datasource_name, connection)
            trans.begin_with_connection()
            return trans
        except DaoException:
            Logger.throw(LogType.DAO, f"开启新事务失败：{name}")




from dataclasses import dataclass, field
from datetime import datetime
from typing import Dict, List, Optional

from mag_db.bean.column import Column


@dataclass
class Table:
    name: Optional[str] = None  # 表名，不含别名
    sn: Optional[int] = None  # 表序号，主键
    schema_name: Optional[str] = None  # 表模式名
    table_type: str = "TABLE"  # 表类型名
    db_sn: Optional[int] = None  # 库序号
    db_name: Optional[str] = None  # 库名
    ds_name: Optional[str] = None  # 数据源名
    alias: Optional[str] = None  # 表别名
    primary_keys: str = ""  # 表的主键清单，逗号分开
    auto_increment: int = 0  # 自增长序号的起始值
    _columns: List[Column] = field(default_factory=list)  # 列名清单
    column_names_mapping: Dict[str, str] = field(default_factory=dict) # 列名与字段名映射表
    create_time: Optional[datetime] = None  # 创建时间
    comment: Optional[str] = None  # 描述

    @property
    def fully_name(self) -> str:
        """
        取完整表名
        """
        return f"{self.name} AS {self.alias}" if self.alias else self.name

    @property
    def columns(self) -> List[Column]:
        return self._columns

    def set_columns(self, columns: List[Column]):
        """
        设置列信息
        :param columns: 列信息清单
        """
        if columns:
            self._columns = columns

            # 设置表的主键：如添加列的信息中包含了是否为主键信息，则设置表的主键
            pri_keys = [col for col in self._columns if col.is_primary_key]
            if pri_keys:
                pri_key_strs = [col.name for col in pri_keys]
                self.primary_keys = ",".join(pri_key_strs)

    def set_column_names(self, column_names: List[str]):
        columns = [Column(name[:-1] if name.endswith('_') else name) for name in column_names]

        self.set_columns(columns)

    def set_sn(self, sn: int):
        """
        设置表序号
        :param sn: 表序号
        """
        self.sn = sn
        for col in self._columns:
            col.table_sn = sn

    def set_primary_keys(self, primary_keys: str):
        """
        设置表的主键信息
        :param primary_keys: 主键清单，逗号分开
        """
        if primary_keys:
            self.primary_keys = primary_keys

            # 更新列的主键信息
            pri_key_strs = primary_keys.split(",")
            for col in self._columns:
                if col.name in pri_key_strs:
                    col.is_primary_key = True

    def __str__(self) -> str:
        """
        流化为完整格式的字符串
        """
        return self.fully_name

    @property
    def uniques(self) -> List[Column]:
        """
        获取有唯一约束的列清单
        :return: 有唯一约束的列清单
        """
        return [col for col in self._columns if col.is_unique]

    def compare(self, table: 'Table') -> bool:
        """
        判定两个表是否相同
        :param table: 表信息
        :return: 是否相同
        """
        return str(self) == str(table)

    def add_column(self, column: Column):
        """
        添加列信息
        :param column: 列信息
        """
        self._columns.append(column)

        # 如该列为主键，但表主键列表中未包含，则添加为主键
        primary_key_strs = self.primary_keys.split(",")
        if column.is_primary_key and column.name not in primary_key_strs:
            primary_key_strs.append(column.name)
            self.primary_keys = ",".join(primary_key_strs)

    def check(self):
        """
        检查表信息格式
        """
        if not self._columns:
            self._columns = []
        if not self.primary_keys:
            self.primary_keys = ""
        if not self.table_type:
            self.table_type = "TABLE"

        for col in self._columns:
            if col.name in self.primary_keys:
                col.is_primary_key = True
                col.is_can_null = False
            col.table_name = self.name

    @classmethod
    def to_tables(cls, table_names: List[str]) -> List['Table']:
        """
        将表名列表转换为表的列表
        :param table_names: 表名的列表
        :return: 表的列表
        """
        return [Table(name=_name) for _name in table_names]